/*******************************************************************************
 * Copyright 2023 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR busINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#ifndef VL53_H
#define VL53_H

#include "voxl_mini_tof_interface.h"

#include "voxl-mini-tof-devices.h"

// The default address of 0x52 is actually 0x29 after a right shift
// since the i2c interface automatically adds the lsb.
#define VL53_DEFAULT_I2C_ADDRESS 0x29
#define VL53_L1_DEFAULT_I2C_ADDRESS 0x29
#define VL53_L7_DEFAULT_I2C_ADDRESS 0x29
#define VL53_L8_DEFAULT_I2C_ADDRESS 0x29


/*
* @brief    This struct holds all the data for a single sensor.
*           It holds a pointer to all available commands and its config
*           data. It also holds its location on a mux chain if applicable.
*/
typedef struct {
    vl53_config * config;
    vl53_commands * commands;
    int mux_channel;    // The channel of the sensor on the mux (ignored if mux_address == 0)
    int mux_address;    // The address of the mux the sensor is connected to (or 0 if none used)
    int bus;
} vl53_sensor;

/*
* @brief    This function takes a list of sensors and starts their
*           constant ranging procedure. The setup will give each
*           sensor a unique address and update the configurations.
*           This MUST be called before getting any data from a sensor
*           will work.
*
* @param    sensors: The list of sensors to use
* @param    num_sensors: The number of sensors in the array
* @return   Returns 0 on success
*/
int start_sequence(tof_config_t enabled_confs[], vl53_sensor sensors[], int n_sensors, int muxes[], int * n_muxes_found);

/*
* @brief    This function is used to get, format, and publish a waiting result
*           from any connected and enabled rangefinder.
*
* @param    sensor: A pointer to the sensor object which is being read
* @param    data: A pointer to the data struct which is prepopulated with
*           data for the given sensor
* @return   Returns 0 on success
*/
int get_data(vl53_sensor * sensor, pt_context * data);

/*
* @brief    Sends the stop ranging command to each sensor.
*
* @param    sensors: The list of sensors to use
* @param    num_sensors: The number of sensors in the array
* @return   Returns 0 on success
*/
int stop_ranging_all(vl53_sensor sensors[], int n_sensors, int muxes[], int n_muxes);

/*
* @brief Generate the list of all enabled sensors from the config structs generated from the config file
*/
int generate_sensors(tof_config_t enabled_sensors[], int n_enabled_sensors, vl53_sensor * sensors);

/*
* @brief    This is used to isolate a single sensor with the multiplexer circuit.
*           It is designed to work with any number of multiplexers and
*           will disable all connections on other multiplexers as well.
*           If no multiplexers are in use it is a no-op and always returns 0.
*
* @param    sensor: The sensor to activate
* @param    muxes: A list of all of the addresses of the muxes
* @param    num_muxes: The number of connected muxes
* @return   Returns 0 on success
*/
int select_one_with_mux(vl53_sensor * sensor, int muxes[], int num_muxes);

#endif //VL53_H